
from typing import Any, List, Dict, Optional, Union, Type, TypeVar, get_origin, get_args

T = TypeVar("T")


class OneOfBaseModel:
    """
    A base class for handling 'oneOf' models where multiple class constructors are available,
    and the appropriate one is determined based on the input data.

    :ivar dict class_list: A dictionary mapping class names to their constructors.
    """

    class_list = {}

    @classmethod
    def return_one_of(cls, input_data: Optional[Any]) -> Optional[Any]:
        """
        Attempts to initialize an instance of one of the classes in the class_list
        based on the provided input data.

        :param input_data: Input data used for initialization.
        :return: An instance of one of the classes specified.
        :rtype: object
        :raises ValueError: If no class can be initialized with the provided input data,
            or if optional parameters don't match the input data.
        """
        if input_data is None:
            return None

        if isinstance(input_data, (str, float, int, bool)):
            return input_data

        exception_list = []
        success_list = []
        for class_constructor in cls.class_list.values():
            try:
                instance = cls._get_instance(class_constructor, input_data)
                if instance is not None:
                    success_list.append(instance)
            except Exception as e:
                exception_list.append({"class": class_constructor, "exception": e})

        if success_list:
            return max(success_list, key=cls._count_non_none_attributes)

        cls._raise_one_of_error(exception_list)

    @classmethod
    def _count_non_none_attributes(cls, instance):
        """
        Count the number of non-None attributes in the instance.

        :param instance: The instance to check.
        :return: The number of non-None attributes.
        """
        if not hasattr(instance, "_map"):
            return 0
        return len([value for value in instance._map().values() if value is not None])

    @classmethod
    def _get_instance(cls, class_constructor, input_data):
        """
        Return an instance of the class based on the input data.

        :param class_constructor: The constructor of the class to check against.
        :param
        :return: An instance of the class if the input data matches the class constructor
            or can be used to initialize the class, None otherwise.
        """
        # Check if the class is only a TypeHint.
        origin = get_origin(class_constructor)
        if origin is not None and isinstance(input_data, list):
            list_instance = cls._get_list_instance(
                input_data, class_constructor, origin
            )
            if list_instance is not None:
                return list_instance
            else:
                return None

        # Check if the input_data is already an instance of the class
        if isinstance(input_data, class_constructor):
            return input_data

        # Check if the input_data is a dictionary that can be used to initialize the class
        elif isinstance(input_data, dict):
            return class_constructor._unmap(input_data)

    @classmethod
    def _get_list_instance(
        cls, input_data: List[Any], class_constructor: Type[T], origin: Type[Any]
    ) -> Union[List[T], List[Any], None]:
        """
        Return the list of elements for a given class constructor and origin type.

        :param input_data: The input data to check.
        :param class_constructor: The constructor of the class to check against.
        :param origin: The origin type to check against.
        :return: The input data if all elements are instances of the type specified in the class_constructor,
            or a new list with each item unmapped.
        """
        args = get_args(class_constructor)
        has_single_arg = args and len(args) == 1

        if not isinstance(input_data, origin) or not has_single_arg:
            return None

        inner_type = args[0]

        if not all(isinstance(item, inner_type) for item in input_data):
            return [inner_type._unmap(item) for item in input_data]

        return input_data

    @classmethod
    def _raise_one_of_error(cls, exception_list):
        """
        Raises a ValueError with the appropriate error message for one of models.

        :param exception_list: List of exceptions that occurred.
        :type exception_list: list
        :raises ValueError: If input data does not match any of the models.
        """
        if not exception_list:
            return
        exception_messages = "\n".join(
            f"Class: {exception['class']}, Exception: {exception['exception']}"
            for exception in exception_list
        )
        raise ValueError(
            f"Input data must match one of the models: {list(cls.class_list.keys())}"
            f"Errors occurred:\n{exception_messages}"
        )




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..listener_status import ListenerStatusService
from ...models import (
    AsyncOperationTokenResult,
    ListenerStatusQueryConfig,
    ListenerStatusAsyncResponse,
)


class ListenerStatusServiceAsync(ListenerStatusService):
    """
    Async Wrapper for ListenerStatusServiceAsync
    """

    def async_get_listener_status(
        self, request_body: ListenerStatusQueryConfig = None
    ) -> Awaitable[Union[AsyncOperationTokenResult, str]]:
        return to_async(super().async_get_listener_status)(request_body)

    def async_token_listener_status(
        self, token: str
    ) -> Awaitable[Union[ListenerStatusAsyncResponse, str]]:
        return to_async(super().async_token_listener_status)(token)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"process_id": "processId", "trading_partner_id": "tradingPartnerId"})
class ProcessingGroupDocumentPartnerRoute(BaseModel):
    """ProcessingGroupDocumentPartnerRoute

    :param process_id: process_id, defaults to None
    :type process_id: str, optional
    :param trading_partner_id: trading_partner_id, defaults to None
    :type trading_partner_id: str, optional
    """

    def __init__(
        self, process_id: str = SENTINEL, trading_partner_id: str = SENTINEL, **kwargs
    ):
        """ProcessingGroupDocumentPartnerRoute

        :param process_id: process_id, defaults to None
        :type process_id: str, optional
        :param trading_partner_id: trading_partner_id, defaults to None
        :type trading_partner_id: str, optional
        """
        if process_id is not SENTINEL:
            self.process_id = process_id
        if trading_partner_id is not SENTINEL:
            self.trading_partner_id = trading_partner_id
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "account_id": "accountId",
        "federation_id": "federationId",
        "id_": "id",
        "user_id": "userId",
    }
)
class AccountUserFederation(BaseModel):
    """AccountUserFederation

    :param account_id: The account ID., defaults to None
    :type account_id: str, optional
    :param federation_id: The federation ID uniquely identifies the user., defaults to None
    :type federation_id: str, optional
    :param id_: The object’s conceptual ID, which is synthesized from the federation, user, and account IDs., defaults to None
    :type id_: str, optional
    :param user_id: The user ID., defaults to None
    :type user_id: str, optional
    """

    def __init__(
        self,
        account_id: str = SENTINEL,
        federation_id: str = SENTINEL,
        id_: str = SENTINEL,
        user_id: str = SENTINEL,
        **kwargs
    ):
        """AccountUserFederation

        :param account_id: The account ID., defaults to None
        :type account_id: str, optional
        :param federation_id: The federation ID uniquely identifies the user., defaults to None
        :type federation_id: str, optional
        :param id_: The object’s conceptual ID, which is synthesized from the federation, user, and account IDs., defaults to None
        :type id_: str, optional
        :param user_id: The user ID., defaults to None
        :type user_id: str, optional
        """
        if account_id is not SENTINEL:
            self.account_id = account_id
        if federation_id is not SENTINEL:
            self.federation_id = federation_id
        if id_ is not SENTINEL:
            self.id_ = id_
        if user_id is not SENTINEL:
            self.user_id = user_id
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .atom_connection_field_extension_summary_simple_expression import (
    AtomConnectionFieldExtensionSummarySimpleExpression,
)
from .atom_connection_field_extension_summary_grouping_expression import (
    AtomConnectionFieldExtensionSummaryGroupingExpression,
)


class AtomConnectionFieldExtensionSummaryExpressionGuard(OneOfBaseModel):
    class_list = {
        "AtomConnectionFieldExtensionSummarySimpleExpression": AtomConnectionFieldExtensionSummarySimpleExpression,
        "AtomConnectionFieldExtensionSummaryGroupingExpression": AtomConnectionFieldExtensionSummaryGroupingExpression,
    }


AtomConnectionFieldExtensionSummaryExpression = Union[
    AtomConnectionFieldExtensionSummarySimpleExpression,
    AtomConnectionFieldExtensionSummaryGroupingExpression,
]




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import AsyncOperationTokenResult, AtomDiskSpaceAsyncResponse


class AtomDiskSpaceService(BaseService):

    @cast_models
    def async_get_atom_disk_space(
        self, id_: str
    ) -> Union[AsyncOperationTokenResult, str]:
        """The GET operation returns the current disk usage state of the given Runtime cloud attachment.
         The initial GET operation returns a token for the specified Runtime cloud attachment. Subsequent GET operations return status code 202 (while the request is in progress) based on the returned token.
         This first request is required to retrieve the authenticating token, which is used in a subsequent GET request.
         >**Note:** `accountId` must always refer to the account ID of the parent Runtime cloud and not that of the attachment.

        :param id_: ID of the Runtime cloud attachment.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AsyncOperationTokenResult, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/async/AtomDiskSpace/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AsyncOperationTokenResult._unmap(response)
        if content == "application/xml":
            return AsyncOperationTokenResult._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def async_token_atom_disk_space(
        self, token: str
    ) -> Union[AtomDiskSpaceAsyncResponse, str]:
        """Send a second HTTP GET request where accountId is the ID of the authenticating account for the request, and token is the token returned in the initial response. This second request authenticates the retrieval of the Runtime cloud attachments' disk space usage.
         >**Note:** `accountId` must always refer to the account ID of the parent Runtime cloud and not that of the attachment.

        :param token: Takes in the token from a previous call to return a result.
        :type token: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AtomDiskSpaceAsyncResponse, str]
        """

        Validator(str).validate(token)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/async/AtomDiskSpace/response/{{token}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("token", token)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AtomDiskSpaceAsyncResponse._unmap(response)
        if content == "application/xml":
            return AtomDiskSpaceAsyncResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .observability_authentication_fields import ObservabilityAuthenticationFields


class AuthType(Enum):
    """An enumeration representing different categories.

    :cvar NONE: "NONE"
    :vartype NONE: str
    :cvar BASIC: "BASIC"
    :vartype BASIC: str
    :cvar BEARER_TOKEN: "BEARER_TOKEN"
    :vartype BEARER_TOKEN: str
    """

    NONE = "NONE"
    BASIC = "BASIC"
    BEARER_TOKEN = "BEARER_TOKEN"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, AuthType._member_map_.values()))


@JsonMap({"auth_type": "authType"})
class ObservabilityEndpointAuthentication(BaseModel):
    """ObservabilityEndpointAuthentication

    :param auth_type: auth_type
    :type auth_type: AuthType
    :param fields: fields, defaults to None
    :type fields: List[ObservabilityAuthenticationFields], optional
    """

    def __init__(
        self,
        auth_type: AuthType,
        fields: List[ObservabilityAuthenticationFields] = SENTINEL,
        **kwargs,
    ):
        """ObservabilityEndpointAuthentication

        :param auth_type: auth_type
        :type auth_type: AuthType
        :param fields: fields, defaults to None
        :type fields: List[ObservabilityAuthenticationFields], optional
        """
        self.auth_type = self._enum_matching(
            auth_type, AuthType.list(), "auth_type"
        )
        if fields is not SENTINEL:
            self.fields = self._define_list(
                fields, ObservabilityAuthenticationFields
            )
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class OutboundValidationOptionCustom(Enum):
    """An enumeration representing different categories.

    :cvar FILTERERROR: "filterError"
    :vartype FILTERERROR: str
    :cvar FAILALL: "failAll"
    :vartype FAILALL: str
    """

    FILTERERROR = "filterError"
    FAILALL = "failAll"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                OutboundValidationOptionCustom._member_map_.values(),
            )
        )


@JsonMap(
    {
        "outbound_message_validation": "outboundMessageValidation",
        "outbound_validation_option": "outboundValidationOption",
        "reject_duplicate_message": "rejectDuplicateMessage",
    }
)
class CustomOptions(BaseModel):
    """CustomOptions

    :param outbound_message_validation: outbound_message_validation, defaults to None
    :type outbound_message_validation: bool, optional
    :param outbound_validation_option: outbound_validation_option, defaults to None
    :type outbound_validation_option: OutboundValidationOptionCustom, optional
    :param reject_duplicate_message: reject_duplicate_message, defaults to None
    :type reject_duplicate_message: bool, optional
    """

    def __init__(
        self,
        outbound_message_validation: bool = SENTINEL,
        outbound_validation_option: OutboundValidationOptionCustom = SENTINEL,
        reject_duplicate_message: bool = SENTINEL,
        **kwargs,
    ):
        """CustomOptions

        :param outbound_message_validation: outbound_message_validation, defaults to None
        :type outbound_message_validation: bool, optional
        :param outbound_validation_option: outbound_validation_option, defaults to None
        :type outbound_validation_option: OutboundValidationOptionCustom, optional
        :param reject_duplicate_message: reject_duplicate_message, defaults to None
        :type reject_duplicate_message: bool, optional
        """
        if outbound_message_validation is not SENTINEL:
            self.outbound_message_validation = outbound_message_validation
        if outbound_validation_option is not SENTINEL:
            self.outbound_validation_option = self._enum_matching(
                outbound_validation_option,
                OutboundValidationOptionCustom.list(),
                "outbound_validation_option",
            )
        if reject_duplicate_message is not SENTINEL:
            self.reject_duplicate_message = reject_duplicate_message
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .trading_partner_processing_group_simple_expression import (
    TradingPartnerProcessingGroupSimpleExpression,
)
from .trading_partner_processing_group_grouping_expression import (
    TradingPartnerProcessingGroupGroupingExpression,
)


class TradingPartnerProcessingGroupExpressionGuard(OneOfBaseModel):
    class_list = {
        "TradingPartnerProcessingGroupSimpleExpression": TradingPartnerProcessingGroupSimpleExpression,
        "TradingPartnerProcessingGroupGroupingExpression": TradingPartnerProcessingGroupGroupingExpression,
    }


TradingPartnerProcessingGroupExpression = Union[
    TradingPartnerProcessingGroupSimpleExpression,
    TradingPartnerProcessingGroupGroupingExpression,
]




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .node_details import NodeDetails


@JsonMap({})
class Nodes(BaseModel):
    """Nodes

    :param node: node, defaults to None
    :type node: List[NodeDetails], optional
    """

    def __init__(self, node: List[NodeDetails] = SENTINEL, **kwargs):
        """Nodes

        :param node: node, defaults to None
        :type node: List[NodeDetails], optional
        """
        if node is not SENTINEL:
            self.node = self._define_list(node, NodeDetails)
        self._kwargs = kwargs




from typing import Any, Dict, Tuple, Generator
from enum import Enum

from .default_headers import DefaultHeaders, DefaultHeadersKeys

from ...net.headers.base_header import BaseHeader

from ...net.transport.request import Request
from ...net.request_chain.request_chain import RequestChain
from ...net.request_chain.handlers.hook_handler import HookHandler
from ...net.request_chain.handlers.http_handler import HttpHandler
from ...net.headers.access_token_auth import AccessTokenAuth
from ...net.headers.basic_auth import BasicAuth
from ...net.request_chain.handlers.retry_handler import RetryHandler


class BaseService:
    """
    A base class for services providing common functionality.

    :ivar str base_url: The base URL for the service.
    :ivar dict _default_headers: A dictionary of default headers.
    """

    def __init__(self, base_url: str) -> None:
        """
        Initializes a BaseService instance.

        :param str base_url: The base URL for the service. Defaults to None.
        """
        self.base_url = base_url
        self._default_headers = DefaultHeaders()
        self._timeout = 60000

        self._update_request_handler()

    def set_access_token(self, access_token: str):
        """
        Sets the access token for the service.
        """
        self._default_headers.set_header(
            DefaultHeadersKeys.ACCESS_AUTH, AccessTokenAuth(access_token)
        )

        return self

    def get_access_token(self) -> BaseHeader:
        """
        Get the access auth header.

        :return: The access auth header.
        :rtype: BaseHeader
        """
        return self._default_headers.get_header(DefaultHeadersKeys.ACCESS_AUTH)

    def set_basic_auth(self, username: str, password: str):
        """
        Sets the username and password for the service.
        """
        self._default_headers.set_header(
            DefaultHeadersKeys.BASIC_AUTH, BasicAuth(username, password)
        )

        return self

    def get_basic_auth(self) -> BaseHeader:
        """
        Get the basic auth header.

        :return: The basic auth header.
        :rtype: BaseHeader
        """
        return self._default_headers.get_header(DefaultHeadersKeys.BASIC_AUTH)

    def set_timeout(self, timeout: int):
        """
        Sets the timeout for the service.

        :param int timeout: The timeout (ms) to be set.
        :return: The service instance.
        """
        self._timeout = timeout
        self._update_request_handler()

        return self

    def set_base_url(self, base_url: str):
        """
        Sets the base URL for the service.

        :param str base_url: The base URL to be set.
        """
        self.base_url = base_url

        return self

    def send_request(self, request: Request) -> Tuple[Dict, int, str]:
        """
        Sends the given request.

        :param Request request: The request to be sent.
        :return: The response data.
        :rtype: Tuple[Dict, int, str]
        """
        response = self._request_handler.send(request)
        return (
            response.body,
            response.status,
            response.headers.get("Content-Type", "").lower(),
        )

    def stream_request(self, request: Request) -> Generator[Dict, None, None]:
        """
        Streams the given request.

        :param Request request: The request to be streamed.
        :return: A generator of the response data.
        :rtype: Generator[Dict, None, None]
        """
        for response in self._request_handler.stream(request):
            yield (
                response.body,
                response.status,
                response.headers.get("Content-Type", "").lower(),
            )

    def get_default_headers(self) -> list:
        """
        Get the default headers.

        :return: A list of the default headers.
        :rtype: list
        """
        return self._default_headers.get_headers()

    def _get_request_handler(self) -> RequestChain:
        """
        Get the request chain.

        :return: The request chain.
        :rtype: RequestChain
        """
        return (
            RequestChain()
            .add_handler(HookHandler())
            .add_handler(RetryHandler())
            .add_handler(HttpHandler(self._timeout))
        )

    def _update_request_handler(self) -> None:
        """
        Update the request handler.
        """
        self._request_handler = self._get_request_handler()




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..process_schedule_status import ProcessScheduleStatusService
from ...models import (
    ProcessScheduleStatus,
    ProcessScheduleStatusBulkResponse,
    ProcessScheduleStatusBulkRequest,
    ProcessScheduleStatusQueryResponse,
    ProcessScheduleStatusQueryConfig,
)


class ProcessScheduleStatusServiceAsync(ProcessScheduleStatusService):
    """
    Async Wrapper for ProcessScheduleStatusServiceAsync
    """

    def get_process_schedule_status(
        self, id_: str
    ) -> Awaitable[Union[ProcessScheduleStatus, str]]:
        return to_async(super().get_process_schedule_status)(id_)

    def update_process_schedule_status(
        self, id_: str, request_body: ProcessScheduleStatus = None
    ) -> Awaitable[Union[ProcessScheduleStatus, str]]:
        return to_async(super().update_process_schedule_status)(id_, request_body)

    def bulk_process_schedule_status(
        self, request_body: ProcessScheduleStatusBulkRequest = None
    ) -> Awaitable[Union[ProcessScheduleStatusBulkResponse, str]]:
        return to_async(super().bulk_process_schedule_status)(request_body)

    def query_process_schedule_status(
        self, request_body: ProcessScheduleStatusQueryConfig = None
    ) -> Awaitable[Union[ProcessScheduleStatusQueryResponse, str]]:
        return to_async(super().query_process_schedule_status)(request_body)

    def query_more_process_schedule_status(
        self, request_body: str
    ) -> Awaitable[Union[ProcessScheduleStatusQueryResponse, str]]:
        return to_async(super().query_more_process_schedule_status)(request_body)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"tag_list_key": "tagListKey"})
class DocCacheProfileParameter(BaseModel):
    """DocCacheProfileParameter

    :param index: index, defaults to None
    :type index: int, optional
    :param key: key, defaults to None
    :type key: int, optional
    :param name: name, defaults to None
    :type name: str, optional
    :param tag_list_key: tag_list_key, defaults to None
    :type tag_list_key: int, optional
    """

    def __init__(
        self,
        index: int = SENTINEL,
        key: int = SENTINEL,
        name: str = SENTINEL,
        tag_list_key: int = SENTINEL,
        **kwargs
    ):
        """DocCacheProfileParameter

        :param index: index, defaults to None
        :type index: int, optional
        :param key: key, defaults to None
        :type key: int, optional
        :param name: name, defaults to None
        :type name: str, optional
        :param tag_list_key: tag_list_key, defaults to None
        :type tag_list_key: int, optional
        """
        if index is not SENTINEL:
            self.index = index
        if key is not SENTINEL:
            self.key = key
        if name is not SENTINEL:
            self.name = name
        if tag_list_key is not SENTINEL:
            self.tag_list_key = tag_list_key
        self._kwargs = kwargs




from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"component_id": "componentId"})
class PublicCertificate(BaseModel):
    """PublicCertificate

    :param alias: alias, defaults to None
    :type alias: str, optional
    :param certificate: certificate, defaults to None
    :type certificate: List[str], optional
    :param component_id: component_id, defaults to None
    :type component_id: str, optional
    """

    def __init__(
        self,
        alias: str = SENTINEL,
        certificate: List[str] = SENTINEL,
        component_id: str = SENTINEL,
        **kwargs
    ):
        """PublicCertificate

        :param alias: alias, defaults to None
        :type alias: str, optional
        :param certificate: certificate, defaults to None
        :type certificate: List[str], optional
        :param component_id: component_id, defaults to None
        :type component_id: str, optional
        """
        if alias is not SENTINEL:
            self.alias = alias
        if certificate is not SENTINEL:
            self.certificate = certificate
        if component_id is not SENTINEL:
            self.component_id = component_id
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .process_property_value import ProcessPropertyValue


@JsonMap(
    {"process_property_value": "ProcessPropertyValue", "component_id": "componentId"}
)
class ProcessProperty(BaseModel):
    """ProcessProperty

    :param process_property_value: process_property_value, defaults to None
    :type process_property_value: List[ProcessPropertyValue], optional
    :param component_id: component_id, defaults to None
    :type component_id: str, optional
    """

    def __init__(
        self,
        process_property_value: List[ProcessPropertyValue] = SENTINEL,
        component_id: str = SENTINEL,
        **kwargs,
    ):
        """ProcessProperty

        :param process_property_value: process_property_value, defaults to None
        :type process_property_value: List[ProcessPropertyValue], optional
        :param component_id: component_id, defaults to None
        :type component_id: str, optional
        """
        if process_property_value is not SENTINEL:
            self.process_property_value = self._define_list(
                process_property_value, ProcessPropertyValue
            )
        if component_id is not SENTINEL:
            self.component_id = component_id
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "attachment_account_id": "attachmentAccountId",
        "attachment_creation_date": "attachmentCreationDate",
        "attachment_instance_id": "attachmentInstanceId",
        "cloud_cluster_id": "cloudClusterId",
        "cloud_id": "cloudId",
        "runtime_id": "runtimeId",
    }
)
class AccountCloudAttachmentSummary(BaseModel):
    """AccountCloudAttachmentSummary

    :param attachment_account_id: attachment_account_id, defaults to None
    :type attachment_account_id: str, optional
    :param attachment_creation_date: attachment_creation_date, defaults to None
    :type attachment_creation_date: str, optional
    :param attachment_instance_id: attachment_instance_id, defaults to None
    :type attachment_instance_id: str, optional
    :param cloud_cluster_id: cloud_cluster_id, defaults to None
    :type cloud_cluster_id: str, optional
    :param cloud_id: cloud_id, defaults to None
    :type cloud_id: str, optional
    :param runtime_id: runtime_id, defaults to None
    :type runtime_id: str, optional
    """

    def __init__(
        self,
        attachment_account_id: str = SENTINEL,
        attachment_creation_date: str = SENTINEL,
        attachment_instance_id: str = SENTINEL,
        cloud_cluster_id: str = SENTINEL,
        cloud_id: str = SENTINEL,
        runtime_id: str = SENTINEL,
        **kwargs,
    ):
        """AccountCloudAttachmentSummary

        :param attachment_account_id: attachment_account_id, defaults to None
        :type attachment_account_id: str, optional
        :param attachment_creation_date: attachment_creation_date, defaults to None
        :type attachment_creation_date: str, optional
        :param attachment_instance_id: attachment_instance_id, defaults to None
        :type attachment_instance_id: str, optional
        :param cloud_cluster_id: cloud_cluster_id, defaults to None
        :type cloud_cluster_id: str, optional
        :param cloud_id: cloud_id, defaults to None
        :type cloud_id: str, optional
        :param runtime_id: runtime_id, defaults to None
        :type runtime_id: str, optional
        """
        if attachment_account_id is not SENTINEL:
            self.attachment_account_id = attachment_account_id
        if attachment_creation_date is not SENTINEL:
            self.attachment_creation_date = attachment_creation_date
        if attachment_instance_id is not SENTINEL:
            self.attachment_instance_id = attachment_instance_id
        if cloud_cluster_id is not SENTINEL:
            self.cloud_cluster_id = cloud_cluster_id
        if cloud_id is not SENTINEL:
            self.cloud_id = cloud_id
        if runtime_id is not SENTINEL:
            self.runtime_id = runtime_id
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .execution_summary_record_expression import (
    ExecutionSummaryRecordExpression,
    ExecutionSummaryRecordExpressionGuard,
)
from .execution_summary_record_simple_expression import (
    ExecutionSummaryRecordSimpleExpression,
)
from .execution_summary_record_grouping_expression import (
    ExecutionSummaryRecordGroupingExpression,
)


@JsonMap({})
class ExecutionSummaryRecordQueryConfigQueryFilter(BaseModel):
    """ExecutionSummaryRecordQueryConfigQueryFilter

    :param expression: expression
    :type expression: ExecutionSummaryRecordExpression
    """

    def __init__(self, expression: ExecutionSummaryRecordExpression, **kwargs):
        """ExecutionSummaryRecordQueryConfigQueryFilter

        :param expression: expression
        :type expression: ExecutionSummaryRecordExpression
        """
        self.expression = ExecutionSummaryRecordExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class ExecutionSummaryRecordQueryConfig(BaseModel):
    """ExecutionSummaryRecordQueryConfig

    :param query_filter: query_filter
    :type query_filter: ExecutionSummaryRecordQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: ExecutionSummaryRecordQueryConfigQueryFilter, **kwargs
    ):
        """ExecutionSummaryRecordQueryConfig

        :param query_filter: query_filter
        :type query_filter: ExecutionSummaryRecordQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, ExecutionSummaryRecordQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from typing import Awaitable
from .utils.to_async import to_async
from ..cancel_execution import CancelExecutionService


class CancelExecutionServiceAsync(CancelExecutionService):
    """
    Async Wrapper for CancelExecutionServiceAsync
    """

    def cancel_execution(self, execution_id: str) -> Awaitable[None]:
        return to_async(super().cancel_execution)(execution_id)




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import AtomCounters


class AtomCountersService(BaseService):

    @cast_models
    def update_atom_counters(
        self, id_: str, request_body: AtomCounters = None
    ) -> Union[AtomCounters, str]:
        """The UPDATE operation updates Runtime Counter values for a specific Runtime. Using the UPDATE operation overrides all settings set on the current counter. However, calling the UPDATE operation does not delete any existing counters that are not included in the `AtomCounters` object.

        :param request_body: The request body., defaults to None
        :type request_body: AtomCounters, optional
        :param id_: A unique ID assigned by the system to the Runtime.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AtomCounters, str]
        """

        Validator(AtomCounters).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AtomCounters/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AtomCounters._unmap(response)
        if content == "application/xml":
            return AtomCounters._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .throughput_account_group_simple_expression import (
    ThroughputAccountGroupSimpleExpression,
)
from .throughput_account_group_grouping_expression import (
    ThroughputAccountGroupGroupingExpression,
)


class ThroughputAccountGroupExpressionGuard(OneOfBaseModel):
    class_list = {
        "ThroughputAccountGroupSimpleExpression": ThroughputAccountGroupSimpleExpression,
        "ThroughputAccountGroupGroupingExpression": ThroughputAccountGroupGroupingExpression,
    }


ThroughputAccountGroupExpression = Union[
    ThroughputAccountGroupSimpleExpression, ThroughputAccountGroupGroupingExpression
]




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({})
class PropertyPair(BaseModel):
    """PropertyPair

    :param encrypted: encrypted, defaults to None
    :type encrypted: bool, optional
    :param key: key, defaults to None
    :type key: str, optional
    :param value: value, defaults to None
    :type value: str, optional
    """

    def __init__(
        self,
        encrypted: bool = SENTINEL,
        key: str = SENTINEL,
        value: str = SENTINEL,
        **kwargs
    ):
        """PropertyPair

        :param encrypted: encrypted, defaults to None
        :type encrypted: bool, optional
        :param key: key, defaults to None
        :type key: str, optional
        :param value: value, defaults to None
        :type value: str, optional
        """
        if encrypted is not SENTINEL:
            self.encrypted = encrypted
        if key is not SENTINEL:
            self.key = key
        if value is not SENTINEL:
            self.value = value
        self._kwargs = kwargs
